@media only screen and (min-width: 1200px) {
    .ui.ui.ui.container:not(.fluid) {
        width: 770px;
        margin-left: auto;
        margin-right: auto;
    }
}

@media only screen and (min-width: 992px) and (max-width: 1200px) {
    .ui.ui.ui.container:not(.fluid) {
        width: 790px;
        margin-left: auto;
        margin-right: auto;
    }
}

@media only screen and (max-width:768px) {
    .ui .grid .three.wide.column {
        width: 100% !important;
        text-align: center;
    }
    .ui .grid .thirteen.wide.column {
        width: 100% !important;
    }
}


body {
    background: #F8F8F8;
    padding: 1em;
}

.logo img{
    max-width: 50px;
    max-height: 50px;
}

footer {
    font-size: x-small;
    text-align: center;
}

footer a {
    color: #B0B0B0;
}

/* Fix button appearance for semantic-ui on webkit */
[type=button] {
    -webkit-appearance: none;
}

.ui.corner.labeled.input .ui.dropdown .dropdown.icon {
    margin-right: 1.5em;
}

i.massive.massive.massive.portrait.icon, i.massive.massive.massive.portrait.icons {
    font-size: 8em;
    margin-bottom: 15px;
}
.profile-form label img{
    cursor: pointer;
}

.profile-form label i{
    cursor: pointer;
}

.dropdown-active {
    background: rgba(0,0,0,.05) !important;
}
